﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Bus
{
    internal class MapsCollection
    {
        readonly Dictionary<string, MapWithSetDoubleDeckerBusGeneric<IDrawingObject, AbstractMap>> _mapStorages;

        public List<string> Keys => _mapStorages.Keys.ToList();

        private readonly int _pictureWidth;

        private readonly int _pictureHeight;

        private readonly char separatorDict = '|';
        
        private readonly char separatorData = ';';

        public MapsCollection(int pictureWidth, int pictureHeight)
        {
            _mapStorages = new Dictionary<string, MapWithSetDoubleDeckerBusGeneric<IDrawingObject, AbstractMap>>();
            _pictureWidth = pictureWidth;
            _pictureHeight = pictureHeight;
        }

        public void AddMap(string name, AbstractMap map)
        {
            if (_mapStorages.ContainsKey(name)) return;
            else
            {
                _mapStorages.Add(name, new MapWithSetDoubleDeckerBusGeneric<IDrawingObject, AbstractMap>(_pictureWidth, _pictureHeight, map));
            }
        }

        public void DelMap(string name)
        { 
            if (_mapStorages.ContainsKey(name))
            {
                _mapStorages.Remove(name);
            }            
        }

        public MapWithSetDoubleDeckerBusGeneric<IDrawingObject, AbstractMap> this[string ind] // инициализация индексатора
        {
            get
            {
                // TODO получает из словаря элемент по ключу
                _mapStorages.TryGetValue(ind, out var result);
                return result;
            }
        }

        private static void WriteToFile(string text, FileStream stream)
        {
            byte[] info = new UTF8Encoding(true).GetBytes(text);
            stream.Write(info, 0, info.Length);
        }

        public void SaveData(string filename)
        {
            if (File.Exists(filename))
            {
                File.Delete(filename);
            }
            using (StreamWriter fs = new(filename))
            {
                fs.Write($"MapsCollection{Environment.NewLine}");
                foreach (var storage in _mapStorages)
                {
                    fs.Write($"{storage.Key}{separatorDict}{storage.Value.GetData(separatorDict, separatorData)}{Environment.NewLine}");
                }
            }
        }

        public void LoadData(string filename)
        {
            if (!File.Exists(filename))
            {
                throw new FileNotFoundException("Файл не найден");
            }
            using (StreamReader sr = new(filename))
            {
                string str = "";
                if ((str = sr.ReadLine()) == null || !str.Contains("MapsCollection"))
                {
                    //если нет такой записи, то это не те данные
                    throw new FileFormatException("Формат данных в файле неправильный");
                }
                //очищаем записи
                _mapStorages.Clear();
                while ((str = sr.ReadLine()) != null)
                {
                    var elem = str.Split(separatorDict);
                    AbstractMap map = null;
                    switch (elem[1])
                    {
                        case "SimpleMap":
                            map = new SimpleMap();
                            break; 
                        case "MyMap":
                            map = new MyMap();
                            break;
                    }
                    _mapStorages.Add(elem[0],new MapWithSetDoubleDeckerBusGeneric<IDrawingObject, AbstractMap>(_pictureWidth, _pictureHeight, map));
                    _mapStorages[elem[0]].LoadData(elem[2].Split(separatorData, StringSplitOptions.RemoveEmptyEntries));
                }
            }
        }
    }
}
