﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Bus
{
    internal class SetDoubleDeckerBusGeneric<T>
        where T : class
    {
        private readonly List<T> _places;
        public int Count => _places.Count;
        private int BusPlaces = 0;

        private readonly int _maxCount;

        public SetDoubleDeckerBusGeneric(int count)
        {
            _maxCount = count;
            _places = new List<T>();
        }

        public int Insert(T bus)
        {
            return Insert(bus, 0);
        }

        private bool isCorrectPosition(int position)
        {
            return 0 <= position && position < _maxCount;
        }

        public int Insert(T bus, int position)
        {
            if (position > _maxCount && position < 0)
            {
                return -1;
            }
            if (_places.Contains(bus))
            {
                throw new ArgumentException($"Объект {bus} уже есть в наборе");
            }
            if (Count == _maxCount)
            {
                throw new StorageOverflowException(_maxCount);
            }
            _places.Insert(position, bus);
            return position;
        }

        public T Remove(int position)
        {
            if (position < 0 || position >= _maxCount)
            {
                return null;
            }
            if (position >= Count || position < 0)
            {
                throw new BusNotFoundException(position);
            }
            var result = _places[position];
            _places.RemoveAt(position);
            return result;
        }

        public T this[int position]
        {
            get
            {
                if (position >= 0 && position < _maxCount && position < Count)
                {
                    return _places[position];
                }
                else
                {
                    return null;
                }
            }
            set
            {
                Insert(value, position);
            }
        }

        public IEnumerable<T> GetBuses()
        {
            foreach (var bus in _places)
            {
                if (bus != null)
                {
                    yield return bus;
                }
                else
                {
                    yield break;
                }
            }
        }
    }
}
